//--------------------------------------------------------------------------------------------------
//
// Title       : ramstate
// Design      : v2
// Author      : Drew Baden
// Company     : University of Maryland
//
//-------------------------------------------------------------------------------------------------
//
// File        : ramstate.v
// Generated   : Tue Jun 21 06:12:43 2005
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------
`timescale 1 ns / 1 ns
//
// dual port ram as histogram
//
module ramstate (
	input clock,
	input nclock,
	input resetn,
	input enable,                 // this is the enable for the FSM
	input ram_enable,			  // turns on the ram output buffers
	input ram_zero,				  // flag to have the ram value zero'd after reading
	input ram_read,				  // if true then we are only reading, otherwise we are also writing
	input[9:0] addr_in,	          // high 10 bits of LHC bucket number
	input[5:0] weight_in,         // value to be added to current value initial addin
	output [15:0] read_value,	  // value in ram memory location pointed to by addr_in
	output reg latch_value,       // use this to latch the value in ram upon reading
	output reg[15:0] sum          // value in ram address addr_in after the sum
	);
	
reg[1:0] state;
parameter[1:0] nWAIT=0, nPAUSE=1, nFETCH=2, nWRITE=3;

reg[5:0] weight_d;
reg[9:0] address_d;
wire[15:0] ram_value;
reg[15:0] value_last;
assign read_value = value_last;
reg ram_write;
reg latchit;
always @ (posedge clock)
	if (~resetn) begin
		state <= nWAIT;
		ram_write <= 0;
		address_d <= 0;
		weight_d <= 0;
		sum <= 0;
		value_last <= 0;
		latchit <= 0;
		end
	else case (state)
		nWAIT: begin
//			if (enable) state <= #1 nFETCH;
			if (enable) state <= #1 nPAUSE;
			else state <= #1 nWAIT;
			ram_write <= #1 0;
			address_d <= #2 addr_in;
			weight_d <= #2 weight_in;
			sum <= #1 0;
			latchit <= #1 0;
			end		 
		//
		// PAUSE is necessary because the ram read and write clocks are out of phase
		//
		nPAUSE: state <= #1 nFETCH;
		nFETCH: begin
			state <= #1 nWRITE;
			value_last <= #2 ram_value;
			ram_write <= #2 0;
			end
		nWRITE: begin
			state <= #1 nWAIT;
			sum <= #1 ram_zero ? 'h0 : weight_d + value_last;
			latchit <= #1 1;
			//
			// this allows us to read without writing, or read and
			// writing a zero in place
			//
			ram_write <= #1 ~ram_read || ram_zero;
			end
		default: begin
			state <= #1 nWAIT;
			end
	endcase	

always @ (posedge clock)
	if (~resetn) latch_value <= 0;
	else latch_value <= #1 latchit;
//
// here is the ram.  we will always read from A and write into B.  there are 4 RAMs, each
// are 1024 long, so we will write into each one sequentially, therefore each RAM will be
// writen into every 4 clocks
//
// ram_enable:    turns on the ram output buffers, not controlled by above FSM
// 
RAMB16_S18_S18 ram_first (
.DOA(ram_value),.DOPA(),.ADDRA(address_d),.CLKA(clock),.DIA(16'b0),.DIPA(2'b0),.ENA(ram_enable),.SSRA(1'b0),.WEA(1'b0),  
.DOB(),.DOPB(),.ADDRB(address_d),.CLKB(nclock),.DIB(sum),.DIPB(2'b0),.ENB(ram_enable),.SSRB(1'b0),.WEB(ram_write)
);						 

endmodule
